# Athena operator service
# lamedrop :: copyright (c) 2003 william pitcock

&log("Setting up protocol events...");

&addRawHook("NICK", \&rawhook_nick);
&addRawHook("MODE", \&rawhook_mode);
&addRawHook("SJOIN", \&rawhook_sjoin);
&addRawHook("PART", \&rawhook_part);
&addRawHook("PRIVMSG", \&rawhook_privmsg);
&addRawHook("NOTICE", \&rawhook_notice);
&addRawHook("KICK", \&rawhook_kick);
&addRawHook("PING", \&rawhook_ping);
&addRawHook("PONG", \&rawhook_pong);
&addRawHook("QUIT", \&rawhook_quit);
&addRawHook("ERROR", \&rawhook_error);

&log("Completed protocol-related initialization.");

# NICK derpeniz 2 1073504433 +i ~derpeniz p508E913A.dip.t-dialin.net irc.endeavortech.net :denpeniz
#  0      1     2      3      4      5             6                          7                8
# 
# :oldnick NICK newnick
#     0     1      2
sub rawhook_nick {
	($line) = @_;
	@data = split (" ", $line, 9);
	if ((scalar @data) == 9) { # New nick.
		$users{@data[1]}{'umode'} = @data[4];
		$users{@data[1]}{'signon'} = @data[3];
		$users{@data[1]}{'user'} = @data[5];
		$users{@data[1]}{'host'} = @data[6];
		$users{@data[1]}{'server'} = @data[7];
		$users{@data[1]}{'gecos'} = @data[8];

		# GECOS pattern matching.
		foreach $pattern (keys %gecos) {
			if ($users{@data[1]}{'gecos'} =~ /\Q$pattern\E/i) {
				&handlexline(@data[1], $pattern);
				return;
			}
		}

		$usercnt++;
		$usercnts{@data[7]}++;
	} else { # Nick change.
		$users{@data[2]}{'umode'} = $users{@data[0]}{'umode'};
		$users{@data[2]}{'signon'} = $users{@data[0]}{'signon'};
		$users{@data[2]}{'user'} = $users{@data[0]}{'user'};
		$users{@data[2]}{'host'} = $users{@data[0]}{'host'};
		$users{@data[2]}{'server'} = $users{@data[0]}{'server'};
		$users{@data[2]}{'gecos'} = $users{@data[0]}{'gecos'};
	}
}

sub rawhook_mode {
	($line) = @_;
	@data = split (" ", $line);
	return if (@data[2] =~ /^#/i);
	$users{@data[2]}{'umode'} = @data[3];
}

sub rawhook_sjoin {
	($line) = @_;
	# remove status symbols from the line, we don't care.
	$line =~ s/\@//g;
	$line =~ s/\+//g;
	$line =~ s/\%//g;

	($null, $chunk1, $chunk2) = split(":", $line);
	($hub, $cmd, $ts, $channel, $modestr) = split(" ", $chunk1, 5);
	@nickdata = split(" ", $chunk2);
	foreach $nick (@nickdata) {
		if ($forbidden{$channel}) {
			&kill($nick, "User joined a drone channel.");
		}
		$chmembers{$channel}{$nick} = $users{$nick}{'host'};
		$chusrcnt{$channel}++;
		if ($onchan{$channel}) {
			# Then op admins
			if ($users{$nick}{'umode'} =~ /a/) {
				print $sock ":Athena MODE $channel +o $nick\r\n";
				next;
			} # and... voice opers
			if ($users{$nick}{'umode'} =~ /o/) {
				print $sock ":Athena MODE $channel +v $nick\r\n";
			}
		}
	}
}

sub rawhook_ping {
	($line) = @_;
	$line =~ s/PING/PONG/g;
	print $sock "$line\n";
}

sub rawhook_pong {
        &log("Next check for lag in 60 seconds.");
        eventadd(time() + 60, "PING :$mynick\n");
}

sub rawhook_kick {
        ($nick1, $command, $chan, $target) = split (" ", $line);
        $nick1 =~ s/^://g;
        $chan =~ s/^://g;
        if ($target eq $mynick) {
                &log("I was kicked from $chan by $nick1. Recovering.");
                print $sock ":$params{'ircServ'} SJOIN ".time()." $chan + $target\n";
		print $sock ":HackServ MODE $chan +o $target\n";
        } else {
                &log("$target was kicked from $chan by $nick1.");
        }
	$chusrcnt{$chan}--;
        if ($params{'auto_limit'} eq "yes") {
                # adjust the user limit.
                if ($chusrcnt{$chan} % 5) {
                        $limit = $chusrcnt{$chan} + 5;
                        if ($nick1 ne $mynick) {
                                print $sock "MODE $chan +l $limit\n";
                        }
                }
        }
        doSignals("channel kick",
                        ("nick" => $target,
                         "kicker" => $nick1,
                         "username" => $username,
                         "hostname" => $hostname,
                         "channel" => $chan));
}

sub rawhook_part {
        ($nick1, $command, $chan, $reason) = split (" ", $line, 4);
        $nick1 =~ s/^://g;
        $chan =~ s/^://g;
	$usermask = $users{$nick1}{'user'}."@".$users{$nick1}{'host'};
        $chusrcnt{$chan}--;
        &log("$nick1 ($usermask) has left $chan. (there are now $chusrcnt{$chan} users)");
        undef $chmembers{$chan}{$nick1};
        doSignals("channel part",
                        ("channel" => $chan,
                         "nick" => $nick1,
                         "username" => $username,
                         "hostname" => $hostname));
} 

sub rawhook_privmsg {
        ($nick1, $command, $chan, $modebuf) = split(" ", $line, 4);
        $nick1 =~ s/^://g;
	if ($nick1 eq "(console)") { return 0; } # Lame hack attempt!
        if ($chan eq $mynick) { $target = $nick1 } else { $target = $chan };
        $cmd = @data[3];
        $arg = @data[4];
        $cmd =~ s/^://g;
        $modebuf =~ s/^://g;
        $cmdline = $modebuf;
        # new hook system! woot!
	if ($target eq $nick1) {
		&log("[$target] $cmdline");
	} else {
		&log("<$nick1/$chan> $cmdline");
	}
	$cmdline =~ s/\001//g;
        &doHooks($nick1, $cmd, $cmdline);
        doSignals("event privmsg",
                        ("target" => $chan,
                         "nick" => $nick1,
                         "username" => $username,
                         "hostname" => $hostname,
			 "message" => $cmdline));
}

sub speak {
	($target, $string) = @_;
	# Messages for the console user should be sent to the console.
	if ($target eq "(console)") {
		&log($string);
		return;
	}
	if ($target eq "(remote)") {
		print $consoletgt "$string\r\n";
		return;
	}
	if ($target eq $chan) {
		&log("<$mynick/$target> $string");
	} else {
		&log("<$mynick> $string");
	}
	print $sock ":$params{'botNick'} PRIVMSG $target :$string\r\n";
}

sub rawhook_quit {
        ($hostmask, $command, $chan, $reason) = split (" ", $line, 4);
        ($nick1, $usermask) = split ("!", $hostmask);
        $nick1 =~ s/^://g;
        $chan =~ s/^://g;
        ($username, $hostname) = split ("@", $usermask);
	foreach $chan (keys %channels) {
		if ($chmembers{$chan}{$nick1}) {
		        $chusrcnt{$chan}--;
		        &log("$nick1 ($usermask) has left $chan. (there are now $chusrcnt{$chan} users)");
		        $chmembers{$chan}{$nick1} = "";
		        if ($params{'auto_limit'} eq "yes") {
		                # adjust the user limit.
		                if ($chusrcnt{$chan} % 5) {
		                        $limit = $chusrcnt{$chan} + 5;
		                        print $sock "MODE $chan +l $limit\n";
		                }
		        }
		}
	}
        doSignals("client quit",
                         ("nick" => $nick1,
                         "username" => $username,
                         "hostname" => $hostname));
	undef $login{$nick1};
}	

sub rawhook_notice {
        ($nick1, $command, $chan, $modebuf) = split(" ", $line, 4);
        $nick1 =~ s/^://g;
        if ($chan eq $mynick) { $target = $nick1 } else { $target = $chan };
        $modebuf =~ s/^://g;
        $cmdline = $modebuf;
        # new hook system! woot!
	if ($nick1 eq "NOTICE") { $nick1 = $params{'linkIP'} };
	&log("-$nick1- $cmdline");
        doSignals("event notice",
                        ("target" => $chan,
                         "nick" => $nick1,
                         "username" => $username,
                         "hostname" => $hostname,
                         "message" => $cmdline));
}

sub rawhook_error {
	undef %chusrcnt;
	&connect();
}

############# COMMANDS #############

sub kill {
	($nick, $reason) = @_;
	print $sock ":$params{'botNick'} KILL $nick :$params{'ircServ'}!$params{'botHost'}!$params{'botUser'}!$params{'botNick'} ($reason)\n";
}

1;
